#!/bin/bash

filename_input="sample_01_Min.mp4"
filename_output="sample_output.mp4"
directory_output="output/$(date +%Y_%m_%d__%H_%M_%S)"

min_threads=1
max_threads=24  # west: 24; abu: 48
len_threads=${#max_threads}

min_queue_length=2
max_queue_length=2
len_queue_length=${#max_queue_length}

run_current=1
run_total=$((($max_threads - $min_threads + 1) * ($max_queue_length - $min_queue_length + 1)))
run_length=${#run_total}

rm -f $filename_output
mkdir -p $directory_output


printf "threads: %d to %d\nqueue length: %d to %d\n" $min_threads $max_threads $min_queue_length $max_queue_length > ${directory_output}/info.txt


clear
printf " %${run_length}s    %${run_length}s  ||  %s  ||  %s  ||  %s  ||  %s\n" " " " " "THREADS" "QUEUE" "⌀FPS" "TIME"
printf -- "----------------------------------------------------------\n"

for (( t=$min_threads; t<=$max_threads; t++ )); do
    for (( ql=$min_queue_length; ql<=$max_queue_length; ql++ )); do
        export OMP_NUM_THREADS=$t

        tf="$(printf "%0${len_threads}d" $t)"  # fill zeros
        qlf="$(printf "%0${len_queue_length}d" $ql)"  # fill zeros
        filename_prefix="${directory_output}/threads${tf}_queue${qlf}"
        filename_stats="${filename_prefix}_stats.txt"
        filename_time="${filename_prefix}_time.txt"

        printf "%s" $filename_prefix > "${directory_output}/current.txt"

        printf " %0${run_length}d of %0${run_length}d  ||  %7s  ||  %5s  ||  " $run_current $run_total $tf $qlf

        (/usr/bin/time --output="$filename_time" --append -f "%e" -- ./soccerAnalyzer "$filename_input" "$filename_output" sample.png $ql) &> "$filename_stats"

        printf "%4s  ||  %4s\n" $(tail -1 "$filename_stats" | cut -d"/" -f4 | rev | cut -d" " -f1 | rev) $(tail -n 1 $filename_time | cut -d"." -f1)
        sleep 5

        rm -f $filename_output
        run_current=$((run_current + 1))

        if (($run_current < $run_total)); then
            sleep 55
        fi 
    done
done

