#!/bin/bash

# set default values
min_threads=1
max_threads=24  # west: 24; abu: 48
min_packet_size=4
max_packet_size=4
min_queue_length=4
max_queue_length=4

filename_input=""
filename_output="/dev/null"
directory_output="output/$(date +%Y_%m_%d__%H_%M_%S)"

while (( "${#}" )); do
    case "${1}" in
        # video input file
        "--vi")
            filename_input="${2}" ;;
        # video output file
        "--vo")
            filename_output="${2}" ;;
        # output file
        "--o")
            xxx="${2}" ;;
        # min threads
        "--tmin")
            min_threads="${2}" ;;
        # max threads
        "--tmax")
            max_threads="${2}" ;;
        # min packet-size
        "--psmin")
            min_packet_size="${2}" ;;
        # max packet-size
        "--psmax")
            max_packet_size="${2}" ;;
        # min queue length
        "--qlmin")
            min_queue_length="${2}" ;;
        # max queue length
        "--qlmax")
            max_queue_length="${2}" ;;
    esac

    shift 2
done

if [ ! -f "$filename_input" ]; then
    printf "Input file '$filename_input' does not exist!!!\n"
    exit 2
fi

len_threads=${#max_threads}
len_queue_length=${#max_queue_length}
len_packet_size=${#max_packet_size}

run_current=1
run_total=$((($max_threads - $min_threads + 1) * ($max_queue_length - $min_queue_length + 1) * ($max_packet_size - $min_packet_size + 1)))
run_length=${#run_total}


cp filename_input /run/shm/
filename_input="/run/shm/${filename_input##*/}"

mkdir -p $directory_output


printf "threads: %d to %d\nqueue length: %d to %d\npackage size: %d to %d\n" $min_threads $max_threads $min_queue_length $max_queue_length $min_packet_size $max_packet_size > ${directory_output}/info.txt


printf " %${run_length}s    %${run_length}s  ||  %s  ||  %s  ||  %s  ||  %s  ||  %s\n" " " " " "THREADS" "QUEUE" "PSIZE" "⌀FPS" "TIME"
printf -- "---------------------------------------------------------------------\n"

for (( t=$min_threads; t<=$max_threads; t++ )); do
    for (( ql=$min_queue_length; ql<=$max_queue_length; ql++ )); do
        for (( ps=$min_packet_size ; ps<=$max_packet_size; ps++ )); do
            export OMP_NUM_THREADS=$t

            tf="$(printf "%0${len_threads}d" $t)"  # fill zeros
            qlf="$(printf "%0${len_queue_length}d" $ql)"  # fill zeros
            psf="$(printf "%0${len_packet_size}d" $ps)"  # fill zeros
            filename_prefix="${directory_output}/threads${tf}_queue${qlf}_packetsize${psf}"
            filename_stats="${filename_prefix}_stats.txt"
            filename_time="${filename_prefix}_time.txt"

            # printf "%s" $filename_prefix > "${directory_output}/current.txt"

            printf " %0${run_length}d of %0${run_length}d  ||  %7s  ||  %5s  ||  %5s  ||  " $run_current $run_total $tf $qlf $psf

            (/usr/bin/time --output="$filename_time" --append -f "%e" -- ./soccerAnalyzer -i "$filename_input" -o "$filename_output" -q $ql -p $ps) &> "$filename_stats"

            printf "%4s  ||  %4s\n" $(tail -1 "$filename_stats" | cut -d"/" -f2 | rev | cut -d" " -f1 | rev) $(tail -n 1 $filename_time | cut -d"." -f1)
            sleep 5

            # rm -f $filename_output
            run_current=$((run_current + 1))

            if (($run_current < $run_total)); then
                sleep 15
            fi 
        done
    done
done
