
#include <stdint.h>
#include <stdbool.h>

#ifndef COLOR_H
#define COLOR_H


// set some basic colors   {r,   g,   b,   a}
static uint8_t black[4]  = {0,   0,   0,   255};
static uint8_t white[4]  = {255, 255, 255, 255};
static uint8_t red[4]    = {255, 0,   0,   255};
static uint8_t green[4]  = {0,   255, 0,   255};
static uint8_t blue[4]   = {0,   0,   255, 255};
static uint8_t yellow[4] = {255, 255, 0,   255};


/**
 * Returns a boolean value whether two colors are identical; ignoring alpha channel
 * @param color_1 first color
 * @param color_2 second color
 * @return true, if two colors are identical
 */
bool colorEqualsRGB(uint8_t * color_1, uint8_t * color_2);

/**
 * Calculates the distance between two colors by euclidean distance as integer; ignoring alpha channel
 * @param color_1 first color
 * @param color_2 second color
 * @return euclidean distance as integer; max value: sqrt(((255^2) * 3)) = floor(441.67296) = 441
 */
uint16_t colorDistanceRGB(uint8_t * color_1, uint8_t * color_2);

/**
 * Sets the color of the specified pixel
 * @param pixel the pixel which color will be set
 * @param color the new color
 */
void colorSetPixelRGBA(uint8_t * pixel, uint8_t * color);


#endif
