
#include "draw.h"
#include "color.h"
#include "math_approx.h"
#include <math.h>
#include <stdlib.h>


// define min and max for integer
#define min(a, b) (((a) < (b)) ? (a) : (b))
#define max(a, b) (((a) > (b)) ? (a) : (b))


void drawLineHorizontal(pictureDataP srcPicture, uint8_t * color, uint16_t row) {
    for (uint16_t col = 0; col < srcPicture->width; col++)
        colorSetPixelRGBA(&(srcPicture->rowPointers[row][col * 4]), color);
}

void drawLineVertical(pictureDataP srcPicture, uint8_t * color, uint16_t col) {
    for (uint16_t row = 0; row < srcPicture->height; row++)
        colorSetPixelRGBA(&(srcPicture->rowPointers[row][col * 4]), color);
}


void drawLine(pictureDataP srcPicture, uint8_t * color, uint16_t pos1_x, uint16_t pos1_y, uint16_t pos2_x, uint16_t pos2_y) {
    double slope = (pos2_y - pos1_y) / (double)(pos2_x - pos1_x);
    
    if (slope <= 1.0 && slope >= -1.0)
        for (uint16_t col = min(pos1_x, pos2_x); col <= max(pos1_x, pos2_x); col++) {
            uint16_t row = (uint16_t)((slope * (col - pos1_x)) + pos1_y);
            colorSetPixelRGBA(&(srcPicture->rowPointers[row][col * 4]), color);
        }
    else
        for (uint16_t row = min(pos1_y, pos2_y); row <= max(pos1_y, pos2_y); row++) {
            uint16_t col = (uint16_t)(((row - pos1_y) / slope) + pos1_x);
            colorSetPixelRGBA(&(srcPicture->rowPointers[row][col * 4]), color);
        }
}


void drawCross(pictureDataP srcPicture, uint8_t * color, uint16_t pos_x, uint16_t pos_y, uint8_t radius) {
    uint8_t diameter = (2 * radius) + 1;

    for (uint8_t i = 1; i <= diameter; ++i)
        for (uint8_t j = 1; j <= diameter; ++j)
            if((i == j && i != diameter) || (i == diameter - j))
                colorSetPixelRGBA(&(srcPicture->rowPointers[pos_y - radius + i][(pos_x - radius + j) * 4]), color);
}


void drawCircle(pictureDataP srcPicture, uint8_t * color, uint16_t pos_x, uint16_t pos_y, uint8_t radius) {
    for (int8_t row = -radius; row <= radius; row++)
        for (int8_t col = -radius; col <= radius; col++)
            if ((approx_squareroot(approx_square(abs(row)) + approx_square(abs(col)))) == radius)
                colorSetPixelRGBA(&(srcPicture->rowPointers[pos_y + row][(pos_x + col) * 4]), color);
}
