
#include "image.h"
#include <stdint.h>

#ifndef DRAW_H
#define DRAW_H


/**
 * draws a line in the specified row
 * @param srcPicture the image which will contain the row
 * @param color the color of the row
 * @param row the row which will the colored
 */
void drawLineHorizontal(pictureDataP srcPicture,    uint8_t * color,    uint16_t row);

/**
 * draws a line in the specified column
 * @param srcPicture the image which will contain the column
 * @param color the color of the column
 * @param col the column which will the colored
 */
void drawLineVertical(  pictureDataP srcPicture,    uint8_t * color,    uint16_t col);

/**
 * draws a straight line between two positions
 * @param srcPicture the image which will contain the new line
 * @param color the color of the line
 * @param pos1_x the x coordinate of the first position
 * @param pos1_y the y coordinate of the first position
 * @param pos2_x the x coordinate of the second position
 * @param pos2_y the y coordinate of the second position
 */
void drawLine(          pictureDataP srcPicture,    uint8_t * color,    uint16_t pos1_x,    uint16_t pos1_y, 
                                                                        uint16_t pos2_x,    uint16_t pos2_y);

/**
 * draws a cross
 * @param srcPicture the image which will contain the cross
 * @param color the color of the cross
 * @param pos_x the x coordinate of the cross
 * @param pos_y the y coordinate of the cross
 * @param radius the radius of the cross; max: 127
 */
void drawCross(         pictureDataP srcPicture,    uint8_t * color,    uint16_t pos_x,     uint16_t pos_y,     uint8_t radius);

/**
 * draws a circle
 * @param srcPicture the image which will contain the circle
 * @param color the color of the circle
 * @param pos_x the x coordinate of the circle
 * @param pos_y the y coordinate of the circle
 * @param radius the radius of the circle; max: 127
 */
 void drawCircle(        pictureDataP srcPicture,    uint8_t * color,    uint16_t pos_x,     uint16_t pos_y,     uint8_t radius);


#endif
