
#include "image.h"
#include <stdbool.h>
#include <glib.h>
#include <libavformat/avformat.h>

#ifndef FFMPEGIO_H
#define FFMPEGIO_H


#define FFMPEG_EOF_REACHED      0x01 < 1
#define FFMPEG_EOF_NOTREACHED   0x01 < 2
#define FFMPEG_EOF_UNKNOWN      0x01 < 3    // If we write a file the EOF is not known


struct ffmpeg {
    AVFormatContext *   fmt_ctx;
    AVCodecContext *    codec_ctx;
    AVStream *          video_stream;
    AVCodec *           video_codec;
    struct SwsContext * sws_ctx;
    uint16_t            stream_index;
    AVFrame *           cur_frame;
    AVPicture           src_frame;
    AVPicture           conv_frame;
    pictureData         image_info;
    AVPacket            cur_packet;
    uint64_t            frame_count;
    uint16_t            width,height;
    uint8_t             eof;
    bool                rescaled;
};

typedef struct ffmpeg ffmpegIOS;
typedef ffmpegIOS * ffmpegIOSP;

extern sig_atomic_t frames;

void ffmpeg_initialize();
uint64_t getFps();
ffmpegIOS *ffmpeg_open_video_input(const char *filename);
ffmpegIOS *ffmpeg_open_video_output(const char *filename, uint16_t width, uint16_t height, uint16_t frame_rate);
int32_t ffmpeg_get_next_frame(ffmpegIOS *video);
int32_t ffmpeg_seek_to_frame(ffmpegIOS *video, uint32_t frame);
int32_t ffmpeg_write_frame(ffmpegIOS *video);
int32_t ffmpeg_close_video(ffmpegIOS *video);


#endif
