
#include "filter.h"
#include <emmintrin.h>


/* Gaussian mask */
static uint8_t gaussianMask2[5][5]  = { 
    {2,  4,  5,  4,  2},
    {4,  9, 12,  9,  4},
    {5, 12, 15, 12,  5},
    {4,  9, 12,  9,  4},
    {2,  4,  5,  4,  2}};

static uint8_t gaussianMask1[3][3] = { 
    {2, 4, 2},
    {4, 9, 4},
    {2, 4, 2}};


// TODO: mask parameter als array und nicht als pointer übergeben 
// TODO: _mm_prefetch in die Hilfsfunktion packen
static void gaussianBlurRGBA(pictureDataP srcPicture, pictureDataP dstPicture, uint8_t matrixSize, uint8_t quotient, uint8_t * mask) {
    for(uint16_t y = 2; y < srcPicture->height - 2; y++)
    for(uint16_t x = 2; x < srcPicture->width  - 2; x++)
    {
        uint64_t sum[4] = {0, 0, 0, 0};

        /* Gaussian matrix multiplication */
        for(int8_t xoff = -matrixSize; xoff <= matrixSize; xoff++)
        for(int8_t yoff = -matrixSize; yoff <= matrixSize; yoff++)
        for(uint8_t i = 0; i < 3; i++)
            sum[i] += srcPicture->rowPointers[y][(x-xoff)*4+i] * mask[(xoff+matrixSize) * (matrixSize*2+1) + (yoff+matrixSize)];
            // sum[i] += srcPicture->rowPointers[y][(x-xoff)*4+i] * mask[xoff+matrixSize][yoff+matrixSize];

        for(uint8_t i = 0; i < 3; i++)
            dstPicture->rowPointers[y][x*4+i] = (uint8_t)(sum[i] / quotient);
    }
}


void gaussianBlur1RGBA(pictureDataP srcPicture, pictureDataP dstPicture) {
    _mm_prefetch (&gaussianMask1, _MM_HINT_NTA);
    gaussianBlurRGBA(srcPicture, dstPicture, 1, 33, &gaussianMask1[0][0]);
}

void gaussianBlur2RGBA(pictureDataP srcPicture, pictureDataP dstPicture) {
    _mm_prefetch (&gaussianMask2, _MM_HINT_NTA);
    gaussianBlurRGBA(srcPicture, dstPicture, 2, 159, &gaussianMask2[0][0]);
}
