
#include "image.h"
#include <libavutil/imgutils.h>


/**
 * @param width 
 * @param height
 * @param color 
 */
pictureDataP image_new(uint16_t width, uint16_t height, enum ColorRange color) {
    pictureDataP data = calloc(sizeof(struct _pictureData),1);

    data->width         = width;
    data->height        = height;
    data->raw           = malloc(width * height * color * sizeof(uint8_t));
    data->rowPointers   = malloc(height * sizeof(uint8_t *));
    data->color         = color;
    av_image_fill_linesizes(data->lineSize,AV_PIX_FMT_RGBA,width);

    for(uint16_t i = 0; i < height ; i++)
        data->rowPointers[i] = &(data->raw[width * i * color]);

    return data;
}

void image_alloc_avPicture(pictureDataP image, enum AVPixelFormat pixfmt)
{
    avpicture_alloc(&image->yuvFrame,pixfmt,image->width,image->height);
}

/**
 * @param width 
 * @param height
 * @param color 
 */
pictureDataP image_new_only_avPicture(uint16_t width, uint16_t height, enum AVPixelFormat pixfmt) {
    pictureDataP data = malloc(sizeof(struct _pictureData));

    data->width         = width;
    data->height        = height;
    data->raw           = NULL;
    data->rowPointers   = NULL;
    data->color         = COLOR_UNKNOWN;

    image_alloc_avPicture(data, pixfmt);

    return data;
}


/**
 * Copy the given Image into a new Buffer.
 * @param ptr The image that should be copied
 */
pictureDataP image_copy(pictureDataP src, pictureDataP dst) {
    struct _pictureData * data;
    if(dst == NULL)
        data =  image_new(src->width,
                          src->height,
                          src->color);
    else
        data = dst;

    memcpy( data->raw, 
            src->raw,
            src->width * src->height * src->color * sizeof(uint8_t));

    return data;
}

void image_free_raw(pictureDataP ptr)
{
    if(ptr->rowPointers != NULL)
        free(ptr->rowPointers);
    if(ptr->raw != NULL)
        free(ptr->raw);
    ptr->raw = NULL;
    ptr->rowPointers = NULL;
}


/**
 * Recursively frees all Buffers used by the given Image
 * @param ptr the image that should be freed
 */
void image_free(pictureDataP ptr) {
    if(ptr == NULL)
        return;
    avpicture_free(&ptr->yuvFrame);
    image_free_raw(ptr);
    free(ptr);
}
