
#include "min-priority-queue.h"


// TODO: Kontrolle, ob Heap leer ist bei touch und dequeue nötig? 
// ANSWER: Da es eine "high" performance queue ist wuerde ein if nur
// weitere branches oder jumps machen.


static void insert(heapNode aNode, heapNode * heap, uint16_t size) {
    uint16_t idx = size + 1;
    heapNode tmp;

    heap[idx] = aNode;

    while (heap[idx].value < heap[idx/2].value && idx > 1) {
        tmp          = heap[idx];
        heap[idx]    = heap[idx/2];
        heap[idx/2]  = tmp;
        idx         /= 2;
    }
}

// TODO: uint16_t bei idx und cidx korrekt?
//       beschreibt lediglich die position im heap, oder?
// ANSWER: ja! sind gespeicherte indexe vom heap um zu sortieren. heapify
// Algorithmus von AD
static void heapify(heapNode * heap, uint16_t size, uint16_t idx) {
    uint16_t cidx;  // index for child
    heapNode tmp;

    for (;;) {
        cidx = idx * 2;

        if (cidx > size)
            break;  // it has no child

        if (cidx < size && heap[cidx].value > heap[cidx+1].value)
            ++cidx;

        //swap if necessary
        if (heap[cidx].value < heap[idx].value) {
            tmp        = heap[cidx];
            heap[cidx] = heap[idx];
            heap[idx]  = tmp;
            idx        = cidx;
        } 
        else 
            break;
    }
}

static inline heapNode removeMin(heapNode * heap, uint16_t size) {
    heapNode rv = heap[1];
    heap[1]     = heap[size];

    --size;
    
    heapify(heap, size, 1);

    return rv;
}

/**
 * This function return the first node in the heap. if size is < 1 this
 * function will return the last item removed from the queue
 * @performance take consistent O(1)
 * @return first node in heap
 * 
 */
heapNode touch(PQ * q) {
    return q->heap[1];
}

/**
 * inserts node in heap. This function doesn't check that this will be
 * a valid operation. You have to check for yourself that the heap is
 * big enough for the next node.
 * @performance Take up to O(log (size))
 * @param q pointer to a min-priority-queue
 * @param node the heapnode that will be inserted
 */
void enqueue(PQ * q, heapNode node) {
    insert(node, q->heap, q->size);
    ++q->size;
}

/**
 * Returns first node and reorganize the heap so the first node is the
 * node with the lowest score. 
 * @performance Take up to O(log (size))
 * @param q pointer to a min-priority-queue there we get the node from
 * @return first node in heap
 */
heapNode dequeue(PQ * q) {
   heapNode rv = removeMin(q->heap, q->size);
   --q->size;

   return rv; 
}

/**
 * Initialize the min priority queue
 * @param q a pointer to a min priority queue struct
 * @param size defines how big the heap structure will be. [0 - 65535
 */
void initializeQueue(PQ * q, uint16_t size) {
   q->size = 0;
   q->heap = (heapNode*) malloc(sizeof(heapNode) * (size + 1));
}
