#include <stdint.h>
#include <stdbool.h>
#include <mpi.h>
#include <glib.h>

#define MPI_JOB_PACKAGE 0xF0
#define MPI_JOB_REQUEST 0xF1
#define MPI_YUV_PACKAGE 0xF3
#define MPI_MAIN_NODE 0x00

/**
 * @brief contains context info for a mpi node
 * we keep track of last frame index send and recv
 * so we can reschedule mpijobs if a node fails
 * 
 */
typedef struct _MpiNode {
    int64_t jobsSend; /**<  */
    int64_t jobsRecv; /**<  */
} MpiNode;

typedef struct _MpiJobRequest {
    uint64_t rank;
    uint64_t num_of_threads;
    uint64_t last_frame_read;
    uint64_t last_frame_writen;
} MpiJobRequest;

/**
 * 
 * @brief descibes a frame interval for a mpijob
 * 
 * 
 */
typedef struct _MpiJob {
    uint64_t first_frame; /**<  this is left closed */
    uint64_t last_frame; /**< this is right open */
} MpiJob;

typedef MpiJobRequest * MpiJobRequestP;
typedef MpiJob * MpiJobP;
typedef MpiNode * MpiNodeP;

MPI_Datatype MPI_JOB_REQUEST_T,
             MPI_JOB_PACKAGE_T;

bool isMpiMainNode();
int32_t getMpiRank();
int32_t getWorldSize();
void initializeMPI(int argc, char * argv[]);
void mpiComm(GAsyncQueue * jobQueue,
             GAsyncQueue * yuvIO,
             uint16_t width, 
             uint16_t heigth,
             size_t   bpp);
void freeMPI();
void increaseMainCounterBy(int i);
