// We are doing gnuc11 This prevent implicit declaration warnings with pthread
#define _GNU_SOURCE
#include <unistd.h>
#include <stdlib.h>
#include <omp.h>
#include <pthread.h>
#include <libswscale/swscale.h>
#include <sys/syscall.h>
#include <sys/resource.h>
#include <getopt.h>
#include "mpicomm.h"
#include "pngedgedetect.h"
#include "pngio.h"
#include "pipeline.h"
#include "min-priority-queue.h"
#include "detect.h"

// queues
uint16_t queue_max_size = 5;
PQ output_mpqueue;              // 
GAsyncQueue * write_queue;      // 

// Process statistics       todo: !!!
uint64_t    wrSi    = 0, // Software interrupts of 
            reSi    = 0, // Software interrupts of 
            prwSiOut= 0, // Software interrupts of 
            prSiOut = 0; // Software interrupts of 


uint32_t write_sleep =200;

GAsyncQueue * myJobs,
            * mpiIn,
            * mpiOut,
            * myJobsOut;

packetProcessP worker;

sig_atomic_t queueFillerExited = false;
sig_atomic_t packetprocessor   = 0;
sig_atomic_t packetsIn         = 0;
sig_atomic_t packetsOut        = 0;

void soccerLog(const char * s, ...) {
    va_list args;
    va_start(args, s);
    fprintf(stderr,"===[%2d]=== ",getMpiRank());
    vfprintf(stderr, s, args);
    fprintf(stderr, "\n");
    va_end(args);
}

void abort_(const char * s, ...) {
    va_list args;
    va_start(args, s);
    vfprintf(stderr, s, args);
    fprintf(stderr, "\n");
    va_end(args);
    abort();
}

// adds packets to the input queue
void * packetQueueFillerThread() {
    pthread_setname_np(pthread_self(), "PQFThread");

    uint64_t packetcounter = 0;

    while(ffmpeg_io_input->eof != FFMPEG_EOF_REACHED)
    {
        // TODO: HIER VON MPI ENTKOPPELN FALLS CODE AUSSERHALB VON MPI
        // GETESTET WERDEN SOLL
        if(getWorldSize() > 1)
        {
            MpiJob * job = g_async_queue_timeout_pop(myJobs,1000000);
            if(job != NULL)
            {
                if(job->first_frame >= ffmpeg_io_input->frame_count)
                {
                    ffmpeg_seek_to_frame(ffmpeg_io_input, job->first_frame);
                    if(!isMpiMainNode())
                        g_async_queue_push(myJobsOut,job);
                    else
                        free(job);
                }
                
                if(ffmpeg_io_input->eof == FFMPEG_EOF_REACHED)
                    break;
            }
            else
                continue;
        }
        
        pipelinePacketP pkg = NULL;
        pkg = pipeline_fill_package(pkg, ffmpeg_io_input);
                
        while(true)
        {
            for(int i = (packetcounter)%omp_get_max_threads() ;
                    i < omp_get_max_threads(); 
                    i++)
            {
                if(!(getWorldSize() > 1 && i == 0) 
                    && g_async_queue_length(worker[i].inqueue) < queue_max_size)
                {
                    g_async_queue_push(worker[i].inqueue, pkg);
                    goto success;
                }
            }
            // If every inputqueue is full, we wait some time.
            // This prevents the thread from runnning at 100% cpu time
            usleep(200);
            reSi++;
        }
        success : 
        packetcounter++;
        packetsIn++;
        continue;
    }  
    soccerLog("PACKAGE QUEUE FILLER EXITED");
    queueFillerExited = true;
}

// prints stats
void * statsThread(void * t) {
    pthread_setname_np(pthread_self(), "wait stats");
    uint64_t overall_fps = 0;
    uint64_t overall_fps_count = 0;
    while (packetprocessor > 0 ||
            !queueFillerExited ||
            packetsIn != packetsOut) {
        overall_fps += frames;    
        overall_fps_count++;
        if(overall_fps_count % 10 == 1)
            soccerLog("STATS: packetsIn \t packetsOut \t Frames(cur) \t"
        " Frames(total) \t Frames(read) \t Frames(writen)");
        soccerLog("\t%8d \t %8d \t %8d/s \t %8d/s \t %8d\t %8d",
        packetsIn, packetsOut,frames,overall_fps/overall_fps_count,
        ffmpeg_io_input->frame_count,ffmpeg_io_output->frame_count);
        if(wrSi > 5000)
        {
            write_sleep += 50 * (wrSi / 2000);
        }
        if(wrSi < 5000)
        {
            write_sleep -= 50;
            if(write_sleep < 50)
                write_sleep = 50;
        }
        wrSi    =
        reSi    =
        prSiOut =
        prwSiOut= 
        frames  = 0;
        sleep(1);
    }
    
    soccerLog("STATS: packetsIn \t packetsOut \t Frames(cur) \t"
        " Frames(total) \t Frames(read) \t Frames(writen)");
    soccerLog("\t%8d \t %8d \t %8d/s \t %8d/s \t %8d\t %8d",
        packetsIn, packetsOut,frames,overall_fps/overall_fps_count,
        ffmpeg_io_input->frame_count,ffmpeg_io_output->frame_count);
}

void * packetProcessThread(packetProcessP context) {
    pthread_setname_np(pthread_self(), "PPThread");
    
    pid_t tid;
    tid = syscall(SYS_gettid);
    int ret = setpriority(PRIO_PROCESS, tid, 15);
    
    /******************************************************************* 
     * We count with an Atomic how many Processor Threads are alive and
     * working and reduce them at the end our main while processing loop
     * 
     * This is used as a Termination condition in other threads
     ******************************************************************/
    packetprocessor++;
    
    while(!queueFillerExited || 
          g_async_queue_length(context->inqueue) > 0) {
        /* We try to get a package that we can process or if the queue
         * is empty we put this thread to sleep. This prevents the 
         * thread from running at 100% CPU */
        pipelinePacketP pkg = g_async_queue_try_pop(context->inqueue);
        if(pkg != NULL)
        {
            /* Here we give the pkg the sws scale contexts, in order to
             * do the scaling from rgb --> yuv in the For each function*/
            pkg->yuv2rgb = context->yuv2rgb;
            pkg->rgb2yuv = context->rgb2yuv;

            pipeline_foreach(pkg, &processFrame);
    
            /* We wait if the Async Queue gets too large until it's smaller
             * than the over the cli params specified maximum size */
            while(g_async_queue_length(context->outqueue) > queue_max_size)
            {
                usleep(50);
                prSiOut++;
            }
            g_async_queue_push(context->outqueue,pkg);
        }
        else
            usleep(20);
    }
    
    packetprocessor--;
}

/**
 * @brief sort function for GList containing PipelinePackages. 
 *        sort list in Ascending id order
 * @param a
 * @param b
 * @return +1 if a > b ; 0 if a == b; -1 if a < b
 */
gint comparePipelinePackage (gconstpointer a,
                             gconstpointer b)
{
    pipelinePacketP pkga = (pipelinePacketP) a;
    pipelinePacketP pkgb = (pipelinePacketP) b;
    
    int id1 = pkga->id;
    int id2 = pkgb->id;
        
    return (id1 > id2 ? +1 : id1 == id2 ? 0 : -1);
}

/**
 * @brief sort function for GList containing MpiJobs. 
 *        sort list in Ascending first_frame order
 * @param a
 * @param b
 * @return +1 if a > b ; 0 if a == b; -1 if a < b
 */
gint compareMPIJob (gconstpointer a,
                    gconstpointer b)
{
    MpiJob * pkga = (MpiJob *) a;
    MpiJob * pkgb = (MpiJob *) b;
    
    int id1 = pkga->first_frame;
    int id2 = pkgb->first_frame;
        
    return (id1 > id2 ? +1 : id1 == id2 ? 0 : -1);
}

/**
 * @brief pops Every gpointer from the queue and insert it into a
 *        sorted List
 * @param queue This Async Queue is empty after calling this function
 * @param list The GList we feed every gpointer into
 * @param sortfunc a Compare function with which we do a 
 *        sorted insert into list
 * @param items a pointer to a int in which we write how many
 *        items we have written into the list
 * @return new GList Pointer.
 */
__attribute__((warn_unused_result))
GList * clearAsyncQueueIntoList(GAsyncQueue * queue, GList * list, 
                                GCompareFunc sortfunc, int * items)
{
    g_async_queue_lock(queue);
    if(g_async_queue_length_unlocked(queue) > 0)
    {
        while(g_async_queue_length_unlocked(queue) > 0)
        {
            list = 
                g_list_insert_sorted(list,
                g_async_queue_pop_unlocked(queue),
                sortfunc);
            if(items)
                (*items)++;
        }
    }
    else
    {
        wrSi++;
    }
    g_async_queue_unlock(queue);
    
    return list;
}

void * packetpreWriteThread() {
    pthread_setname_np(pthread_self(), "PRWThread");
    GList * holdbackpackets = NULL; /**< In this list we store PipelinePackets 
                                      * which can not be written at this time, 
                                      * because we need to ensure we write them 
                                      * in the right order */
    GList * mpijobsOut = NULL;
         
    while(  packetprocessor > 0 ||
            !queueFillerExited ||
            holdbackpackets != NULL ||
            packetsIn != packetsOut)
    {
        for(int i = 0; i < omp_get_max_threads();i++)
        {
            int items =0;
            holdbackpackets = 
                clearAsyncQueueIntoList(worker[i].outqueue,
                                        holdbackpackets,
                                        comparePipelinePackage,
                                        &items);
            if(isMpiMainNode() && getWorldSize() > 1)
                increaseMainCounterBy(items);
        }
        
        if(isMpiMainNode())
        {
            holdbackpackets = 
                clearAsyncQueueIntoList(mpiIn, 
                                        holdbackpackets,
                                        comparePipelinePackage,
                                        NULL);
        }
        else
        {
            /* We Store mpiJob packages in a sorted List to ensure we send
             * our Pipelinepackets in the order we got our mpiJobs */
             mpijobsOut = clearAsyncQueueIntoList(myJobsOut, 
                                                  mpijobsOut,
                                                  compareMPIJob,
                                                  NULL);
        }
        
        while(holdbackpackets != NULL &&
            (mpijobsOut != NULL || isMpiMainNode()) )
        {
            pipelinePacketP pkg = g_list_first(holdbackpackets)->data;
            if(isMpiMainNode())
            {   
                if(pkg->id == ffmpeg_io_output->frame_count)
                {
                    packetsOut++;
                    g_async_queue_push(write_queue, pkg);
                    holdbackpackets = g_list_remove(holdbackpackets,pkg);
                }
                else
                    break;
            }
            else
            {
                MpiJobP job = g_list_first(mpijobsOut)->data;
                if(job->first_frame == pkg->id)
                {
                    packetsOut++;
                    g_async_queue_push(mpiOut, pkg);
                    holdbackpackets = g_list_remove(holdbackpackets,pkg);
                    mpijobsOut = g_list_remove(mpijobsOut,job);
                    free(job);
                }
                else
                    break;
            }
        }

        
        usleep(write_sleep);
    }

    g_list_free_full(holdbackpackets, (void (*)(void *)) pipeline_free_package);
      
}

void * packetWriteThread() {
    pthread_setname_np(pthread_self(), "PWThread");

    while(  packetprocessor > 0 ||
            !queueFillerExited ||
            packetsIn != packetsOut ||
            g_async_queue_length(write_queue) > 0 ||
            ffmpeg_io_input->eof == FFMPEG_EOF_NOTREACHED) {
        pipelinePacketP pkg = g_async_queue_pop(write_queue);   // blocks until data gets available
        pipeline_write_packet(pkg, ffmpeg_io_output);
        pipeline_free_package(pkg);
    }
}

void usage(char * binName)
{
    printf(
"usage: %s -i <input.mp4> -o <output.flv> -p <[0-65535]>\n"
"\n"
"Options:\n"
"  -i, --input_file <input.ext>     File name of Input, every format\n"
"                                   thats supported by ffmpeg is valid\n"
"  -o, --output_file <output.flv>   File name of output. If the file exists\n"
"                                   it will be overwritten\n"
"  -p, --package_size [0-65535]     Defines how big Frame Packages will be\n"
"                                   Affects total FPS performance. With a high\n"
"                                   Number of Threads consider increasing it\n"
"                                   A Good starting point is Threads * 2\n"
"  -q, --queue_size [0-65535]       This sets the maximum size of Package\n"
"                                   Queue Buffers. This has a huge inpact\n"
"                                   in the memory consumption of this programm\n"
"                                   Use with caution\n"
"  -e, --h264_encode_threads [0-48] Specifies how many Encoding thread the\n"
"                                   h264 codec will use\n"
"  -e, --h264_encode_threads [0-48] Specifies how many Decoding thread the\n"
"                                   h264 codec will use\n"
"  -h, --help\n"
"  -u, --usage                      Prints this help information\n",binName);
}

char * inputFileName    = 0;
char * outputFileName   = 0;
char * sampleFileName   = 0;

void parseCLICommands(int argc, char * argv[]) {
    int c;
    int digit_optin = 0;
    
    while(1)
    {
        int this_option_optind = optind ? optind : 1;
        int option_index =0;
        static struct option long_options[] =
        {
            {"input-file",         required_argument, 0, 'i'},
            {"output_file",        required_argument, 0, 'o'},
            {"sample_img",         required_argument, 0, 's'},
            {"package_size",       required_argument, 0, 'p'},   
            {"queue_size",         required_argument, 0, 'q'},
            {"h264_encode_threads",required_argument, 0, 'e'},
            {"h264_decode_threads",required_argument, 0, 'd'},
            {"help",               no_argument,       0, 'h'},
            {"usage",              no_argument,       0, 'u'},      
            {0,                    0,                 0,  0 }
        };
        
        c = getopt_long(argc, argv,"i:o:s:p:q:e:d:hu", 
                        long_options, &option_index);
                        
        if(c == -1)
            break;
        
        switch(c)
        {
        case 'i':
            inputFileName = optarg;
            break;
        case 'o':
            outputFileName = optarg;
            break;
        case 's':
            sampleFileName = optarg;
            break;
        case 'p':
            pipeline_set_max_size(atoi(optarg));
            break;
        case 'q':
            queue_max_size = atoi(optarg);
            break;
        case 'e':
            break;
        case 'd':
            break;
        case 'h':
        case 'u':
        default:
            usage(argv[0]);
            exit(EXIT_SUCCESS);
        }
    }
}

void initializeWorker() {
    worker = calloc(omp_get_max_threads(), sizeof(struct packetProcess));

    for(int i = 0; i < omp_get_max_threads(); i++) {
        worker[i].inqueue  = g_async_queue_new();
        worker[i].outqueue = g_async_queue_new();
        worker[i].yuv2rgb  = sws_getContext(ffmpeg_io_input->width, 
                                            ffmpeg_io_input->height, 
                                            ffmpeg_io_input->codec_ctx->pix_fmt, 
                                            ffmpeg_io_input->width, 
                                            ffmpeg_io_input->height, 
                                            AV_PIX_FMT_RGBA,    0, 0, 0, 0);
        worker[i].rgb2yuv  = sws_getContext(ffmpeg_io_input->width, 
                                            ffmpeg_io_input->height, 
                                            AV_PIX_FMT_RGBA, 
                                            ffmpeg_io_input->width, 
                                            ffmpeg_io_input->height, 
                                            AV_PIX_FMT_YUV420P, 0, 0, 0, 0);
    }
}

void freeWorker()
{
    for(int i = 0; i < omp_get_max_threads(); i++)
    {
        sws_freeContext(worker[i].yuv2rgb);
        sws_freeContext(worker[i].rgb2yuv);
        g_async_queue_unref(worker[i].inqueue);
        g_async_queue_unref(worker[i].outqueue);
    }
    free(worker);
}

void readFiles()
{
    if(inputFileName == NULL)
        abort_("No input file specified");
    if(outputFileName == NULL)
        abort_("No output file specified");
    ffmpeg_io_input  = ffmpeg_open_video_input(inputFileName);
    if(isMpiMainNode())
        ffmpeg_io_output = ffmpeg_open_video_output(outputFileName,
                                                    ffmpeg_io_input->width,
                                                    ffmpeg_io_input->height,
                                                    45);
}

void printHeader()
{
    soccerLog("NUM OF ThREADS %d",omp_get_max_threads());
}

void initializeAsyncQueues()
{
    write_queue = g_async_queue_new();
    myJobs      = g_async_queue_new();
    mpiIn       = g_async_queue_new();
    mpiOut      = g_async_queue_new();
    myJobsOut   = g_async_queue_new();
}

int main(int argc, char * argv[]) 
{
    parseCLICommands(argc, argv);

    ffmpeg_initialize();
    readFiles();
    initializeMPI(argc, argv);
    initializeAsyncQueues();
    initializeDetect(&(ffmpeg_io_input->image_info));
    initializeWorker();

    MPI_Barrier(MPI_COMM_WORLD);

    pthread_t write;
    pthread_t read;
    pthread_t stats;
    pthread_t preWrite;
    
    pthread_create(&read,  NULL, packetQueueFillerThread, NULL);
    pthread_create(&preWrite, NULL, packetpreWriteThread, NULL);

    if(myRank == 0) {
        pthread_create(&stats, NULL, statsThread,       NULL);
        pthread_create(&write, NULL, packetWriteThread, NULL);
    }

    // start the process on all threads
    
    #pragma omp parallel shared(statistical_border_tb_array, statistical_border_lr_array)
    {
        // TODO: HIER VON MPI ENTKOPPELN FALLS CODE AUSSERHALB VON MPI
        // GETESTET WERDEN SOLL 
        if(omp_get_thread_num() == 0 && getWorldSize() > 1)
        {
            mpiComm(myJobs,
                    (isMpiMainNode() ? mpiIn : mpiOut),
                    ffmpeg_io_input->width,
                    ffmpeg_io_input->height,12); 
        }
        else
        {
            packetProcessThread(&worker[omp_get_thread_num()]);
        }
    }

    pthread_join(read,NULL);
    pthread_join(preWrite,NULL);
    
    if(myRank ==0)
    {
        pthread_join(stats,NULL);
        pthread_join(write,NULL);
    }
    // close the video files
    ffmpeg_close_video(ffmpeg_io_input);
    if(isMpiMainNode())
        ffmpeg_close_video(ffmpeg_io_output);
    freeWorker();
    freeMPI();
    finalizeDetect();
    
}
