#include <stdint.h>
#include <stdlib.h>
#include "pngio.h"
#include "pngedgedetect.h"

pictureDataP png_read_file(char *file_name)
{
    char header[8];    // 8 is the maximum size that can be checked

    /* open file and test for it being a png */
    FILE *fp = fopen(file_name, "rb");
    if (!fp)
            abort_("[png_read_file] File %s could not be opened for reading", file_name);
    size_t ret = fread(header, 1, 8, fp);
    if (ret == 8 && png_sig_cmp(header, 0, 8))
            abort_("[png_read_file] File %s is not recognized as a PNG file", file_name);


    /* initialize stuff */
    png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);

    if (!png_ptr)
            abort_("[png_read_file] png_create_read_struct failed");

    png_infop info_ptr = png_create_info_struct(png_ptr);
    if (!info_ptr)
            abort_("[png_read_file] png_create_info_struct failed");

    if (setjmp(png_jmpbuf(png_ptr)))
            abort_("[png_read_file] Error during init_io");
    
    png_init_io(png_ptr, fp);
    png_set_sig_bytes(png_ptr, 8);

    png_read_info(png_ptr, info_ptr);

    imgCRange range = COLOR_UNKNOWN;
    if(png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGBA)
        range = COLOR_RGBA;
        
    pictureDataP picture =
            image_new(png_get_image_width(png_ptr, info_ptr),
                      png_get_image_height(png_ptr, info_ptr),
                      range);
    png_infop iinfo_ptr = png_create_info_struct(png_ptr);
    png_read_update_info(png_ptr, info_ptr);

    
    /* read file */
    if (setjmp(png_jmpbuf(png_ptr)))
            abort_("[png_read_file] Error during read_image");

    png_read_image(png_ptr, picture->rowPointers);
    
    png_destroy_read_struct(&png_ptr,&info_ptr,&iinfo_ptr);
    
    fclose(fp);
    
    return picture;
}

void png_write_file(pictureDataP handle, char *file_name)
{
    /* create file */
    FILE *fp = fopen(file_name, "wb");
    if (!fp)
            abort_("[png_write_file] File %s could not be opened for writing", file_name);


    /* initialize stuff */
    png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);

    if (!png_ptr)
            abort_("[png_write_file] png_create_write_struct failed");

    png_infop info_ptr = png_create_info_struct(png_ptr);
    if (!info_ptr)
            abort_("[png_write_file] png_create_info_struct failed");

    if (setjmp(png_jmpbuf(png_ptr)))
            abort_("[png_write_file] Error during init_io");

    png_init_io(png_ptr, fp);


    /* write header */
    if (setjmp(png_jmpbuf(png_ptr)))
            abort_("[png_write_file] Error during writing header");

    png_set_IHDR(png_ptr, info_ptr, handle->width, handle->height,
                 8 , PNG_COLOR_TYPE_RGBA, PNG_INTERLACE_NONE,
                 PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);

    png_write_info(png_ptr, info_ptr);


    /* write bytes */
    if (setjmp(png_jmpbuf(png_ptr)))
            abort_("[png_write_file] Error during writing bytes");

    png_write_image(png_ptr,(png_bytepp) handle->rowPointers);


    /* end write */
    if (setjmp(png_jmpbuf(png_ptr)))
            abort_("[png_write_file] Error during end of write");

    png_write_end(png_ptr, NULL);

    /* cleanup heap allocation */
    png_destroy_write_struct(&png_ptr, &info_ptr);

    fclose(fp);
}
